
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

#import <time.h>
time_t time(time_t *t);

#import "Controller.h"
#import "Document.h"
#import "KnobView.h"
#import "DigitSource.h"

@implementation Controller

static NSPanel *_prog_ind_panel = nil;

#define PROG_IND_WIDTH 200
#define PROG_IND_HEIGHT 20

+ (NSPanel *)prog_ind
{
    if(_prog_ind_panel==nil){
	NSRect pi_frame =
	    NSMakeRect(0, 0, PROG_IND_WIDTH, PROG_IND_HEIGHT);

        _prog_ind_panel = 
            [[NSPanel alloc]
                initWithContentRect:pi_frame
                styleMask:NSTitledWindowMask
                backing:NSBackingStoreBuffered
                defer:NO];
        [_prog_ind_panel setReleasedWhenClosed:NO];
        
	KnobView *_prog_ind =
	    [[KnobView alloc] initWithFrame:pi_frame];
        [_prog_ind_panel setContentView:_prog_ind];

	[_prog_ind_panel setTitle:_(@"Computing Sudoku")];
    }

    [_prog_ind_panel center];
    return _prog_ind_panel;
}

#define TICK_ITER 20
#define TICK_STEP  5

- newPuzzle:(id)sender; // clues = [sender tag]
{
    NSApplication *app = [NSApplication sharedApplication];

    NSDocumentController *dc =
	[NSDocumentController sharedDocumentController];
    [dc newDocument:self];

    Document *doc = [dc currentDocument];
    Sudoku *sdk = [doc sudoku];

    NSPanel *pi_panel = [Controller prog_ind];
    KnobView *pi = [pi_panel contentView];
    
    [pi_panel makeKeyAndOrderFront:self];

    NSModalSession findSession;
    findSession = [app beginModalSessionForWindow:pi_panel];

    float percent = 0, dir = 1;

    do {
	int tick;
	for(tick=0; tick<TICK_ITER; tick++){
	    [pi setPercent:percent];
	    [pi display];

	    [[pi_panel contentView] setNeedsDisplay:YES];
	    [pi_panel flushWindow];

	    percent += TICK_STEP*dir;
	}
	if(percent==100.0){
	    dir = -1;
	}
	else if(percent==0.0){
	    dir = +1;
	}
	[app runModalSession:findSession];
    } while([sdk find:(unsigned long)time(NULL)]==NO);

    [pi_panel orderOut:self];
    [app endModalSession:findSession];

    int clues = [sender tag];
    [sdk setClues:clues];

    [sdk setNeedsDisplay:YES];
    [sdk display];

    [doc updateChangeCount:NSChangeDone];

    return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
{
  NSDocumentController *dc = 
    [NSDocumentController sharedDocumentController];

  // Make the DocumentController the delegate of the application.
  // [NSApp setDelegate: dc];

  NSFileManager *fm = [NSFileManager defaultManager];
  NSArray *procArgs = [[NSProcessInfo processInfo] arguments];
  int arg;

  for(arg=1; arg<[procArgs count]; arg++){  // skip program name
    NSString *docFile = [procArgs objectAtIndex:arg];

    if([fm isReadableFileAtPath:docFile]==NO){
        NSLog(@"couldn't open %@ for reading", docFile);
    }
    else{
        NSArray *comps = [docFile pathComponents];
        if([[comps objectAtIndex:0] isEqualToString:@"/"]==NO){
            docFile = 
                [NSString stringWithFormat:@"%@/%@",
                          [fm currentDirectoryPath], docFile];
        }

        [dc openDocumentWithContentsOfFile:docFile display:YES];
    }
  }

  [self makeDigitPalette];
}

- makeDigitPalette
{
  NSRect pbounds = 
    NSMakeRect(0, 0, 2*DIGIT_FIELD_DIM, 5*DIGIT_FIELD_DIM);

  NSRect 
    pframe =
    [NSWindow frameRectForContentRect:pbounds
	      styleMask:NSTitledWindowMask];

  NSPanel *palette =
    [[NSPanel alloc] initWithContentRect:pframe 
		      styleMask:NSTitledWindowMask                   
		      backing: NSBackingStoreRetained 
		      defer:YES];

  [palette setMinSize:pframe.size];
  [palette setMaxSize:pframe.size];

  [palette setTitle:_(@"Digits")];
  [palette setDelegate:self];

  [palette setFrameUsingName: @"SudokuDigitPalette"];
  [palette setFrameAutosaveName: @"SudokuDigitPalette"];


  int x, y;
  for(x=0; x<2; x++){
    for(y=0; y<5; y++){
      DigitSource *dgs =
	[[DigitSource alloc] 
	  initAtPoint:
	    NSMakePoint(x*DIGIT_FIELD_DIM, (4-y)*DIGIT_FIELD_DIM)
	  withDigit:y*2+x+1];
      [[palette contentView] addSubview:dgs];
    }
  }

  [palette orderFrontRegardless];
  [palette makeKeyWindow];

  return self;
}

@end


