#import "Document.h"

@interface Document (Private)

- (NSWindow*)makeWindow;

@end

@implementation Document

- init
{
  [super init];

  sdk = nil; lines = nil;
  return self;
}

- (NSData *)dataRepresentationOfType:(NSString *)aType 
{
    if([aType isEqualToString:DOCTYPE]){
        [[sdk window] saveFrameUsingName:[self fileName]];

        NSString *all =
	    [NSString stringWithFormat:@"%@\n%@\n%@\n%@\n",
		      [sdk stateToString:FIELD_VALUE],
		      [sdk stateToString:FIELD_PUZZLE],
		      [sdk stateToString:FIELD_GUESS],
		      [sdk stateToString:FIELD_SCORE]];

        return [all dataUsingEncoding:NSASCIIStringEncoding];
    }
    else{
        NSString *msg = [NSString stringWithFormat: @"Unknown type: %@", 
                        [aType uppercaseString]];
        NSRunAlertPanel(@"Alert", msg, @"Ok", nil, nil);
        return nil;
    }
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType 
{
    if([aType isEqualToString:DOCTYPE]){
        lines = 
            [[NSString stringWithCString:[data bytes] 
                       length:[data length]] 
                componentsSeparatedByString:@"\n"];

	RETAIN(lines);
    }
    else{
        NSString *msg = [NSString stringWithFormat: @"Unknown type: %@", 
                        [aType uppercaseString]];
        NSRunAlertPanel(@"Alert", msg, @"Ok", nil, nil);
        return NO;
    }

    return YES;
}

- (void) makeWindowControllers
{
  NSWindowController *controller;
  NSWindow *win = [self makeWindow];
  
  controller = [[NSWindowController alloc] initWithWindow: win];
  RELEASE (win);
  [self addWindowController: controller];
  RELEASE(controller);

  // We have to do this ourself, as there is currently no nib file
  // [controller setShouldCascadeWindows:NO];
  [self windowControllerDidLoadNib: controller];

  [win setFrameAutosaveName:[self fileName]];
  if([win setFrameUsingName:[self fileName]]==NO){
      [win center];
  }

  [win orderFrontRegardless];
  [win makeKeyWindow];
  [win display];
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController;
{
  [super windowControllerDidLoadNib:aController];

  NSEnumerator *en = [lines objectEnumerator];

  if(lines != nil){
    [sdk stateFromLineEnumerator:en what:FIELD_VALUE];
    [sdk stateFromLineEnumerator:en what:FIELD_PUZZLE];
    [sdk stateFromLineEnumerator:en what:FIELD_GUESS];
    [sdk stateFromLineEnumerator:en what:FIELD_SCORE];
    
    RELEASE(lines); lines = nil;
  }
}

- (Sudoku *)sudoku
{
    return sdk;
}

- resetPuzzle:(id)sender
{
  [sdk reset];
  [self updateChangeCount:NSChangeDone];

  return self;
}

@end

@implementation Document (Private)

- (NSWindow*)makeWindow
{
  NSWindow *window;
  int m = (NSTitledWindowMask |  
	   NSClosableWindowMask | 
           NSMiniaturizableWindowMask);

  NSRect frame = {{ 0, 0}, {SDK_DIM, SDK_DIM} };
  sdk  = [[Sudoku alloc] initWithFrame:frame];

  frame = [sdk frame]; // just in case

  window = 
      [[NSWindow alloc] initWithContentRect:frame 
			styleMask:m                   
			backing: NSBackingStoreRetained 
                             defer:YES];
  [window setDelegate:self];

  [window setContentView:sdk];
  [window setReleasedWhenClosed:YES];

  [self setFileType:DOCTYPE];
  
  return window;
}

@end
