
#import "Sudoku.h"
#import "DigitSource.h"

@implementation Sudoku

- (int)computescore:(fieldptr)fp
{
    int d, res = 0;

    for(d=0; d<9; d++){
	if(fp->nbdigits[d] != NULL){
	    res++;
	}
    }
 
    fp->score = res;
    return res;
}

#define RETR(_x, _y) \
(data[_x][_y].puzzle==-1 ? data[_x][_y].guess : data[_x][_y].puzzle)

- (BOOL)completed
{
  int x, y;
  
  for(x=0; x<9; x++){
    for(y=0; y<9; y++){
      if(RETR(x, y)==-1){
	return NO;
      }
    }
  }
   
  return YES;
}

- initWithFrame:(NSRect)frame
{
    frame.size.width = SDK_DIM;
    frame.size.height = SDK_DIM;

    [super initWithFrame:frame];

    [self registerForDraggedTypes:
	    [NSArray arrayWithObjects:DIGIT_TYPE, nil]];

    clues = -1;

    int x, y, pos;

    for(x=0; x<9; x++){
      for(y=0; y<9; y++){
	data[x][y].value = -1;
	data[x][y].guess = -1;
	data[x][y].puzzle = -1;
      }
    }

    // adjaceny
    for(x=0; x<9; x++){
	for(y=0; y<9; y++){
	    int n = 0;

	    for(pos=0; pos<9; pos++){
		if(pos!=y){
		    data[x][y].adj[n].nx = x;
		    data[x][y].adj[n].ny = pos;

		    n++;
		}
	    }

	    for(pos=0; pos<9; pos++){
		if(pos!=x){
		    data[x][y].adj[n].nx = pos;
		    data[x][y].adj[n].ny = y;

		    n++;
		}
	    }
      
	    int zx = x/3, zy = y/3;
	    for(pos=0; pos<9; pos++){
		int zzx = 3*zx + pos/3, zzy = 3*zy + pos%3; 
		if(zzx!=x && zzy!=y){
		    data[x][y].adj[n].nx = zzx;
		    data[x][y].adj[n].ny = zzy;

		    n++;
		}
	    }

	    assert(n == NBCOUNT);
	}
    }

    return self;
}


- (NSString *)stateToString:(FIELD_TYPE)what
{
    NSString *res = @"";

    int x, y;
    
    for(y=0; y<9; y++){
        for(x=0; x<9; x++){
	    int outval;
	    if(what==FIELD_VALUE){
		outval = data[x][y].value;
	    }
	    else if(what==FIELD_PUZZLE){
		outval = data[x][y].puzzle;
	    }
	    else if(what==FIELD_GUESS){
		outval = data[x][y].guess;
	    }
	    else{
		[self computescore:&(data[x][y])];
		outval = data[x][y].score;
	    }

            res = 
                [res stringByAppendingFormat:
                         (x>0 ? @" %c" : @"%c"),
                     (outval==-1 ? '.' : 
		      ((what==FIELD_SCORE ? '0' : '1') + outval))];
        }
        res = [res stringByAppendingString:@"\n"];
    }

    return res;
}

- stateFromLineEnumerator:(NSEnumerator *)en what:(FIELD_TYPE)what
{
    int x, y;

    for(y=0; y<9; y++){
	NSString *line = [en nextObject];
	NSArray *fields = 
	  [line componentsSeparatedByString:@" "];
	NSEnumerator *fieldenum = [fields objectEnumerator];

        for(x=0; x<9; x++){
	  NSString *field = [fieldenum nextObject];
	  NSScanner *scn = 
	    [NSScanner scannerWithString:field];

	  int inval = -1;
	  if([field isEqualToString:@"."]==NO){
	    [scn scanInt:&inval];
	  }

	  if(what==FIELD_VALUE){
	    data[x][y].value = inval-1;
	  }
	  else if(what==FIELD_PUZZLE){
	    data[x][y].puzzle = 
	      (inval==-1 ? -1 : inval-1);
	  }
	  else if(what==FIELD_GUESS){
	    data[x][y].guess = 
	      (inval==-1 ? -1 : inval-1);
	  }
	  else{
	    data[x][y].score = inval;
	  }
	}
    }

    // skip separator
    // NSLog(@"sep <%@>", [en nextObject]);
    [en nextObject];
    return self;
}

#define RAND_DIST 35

- (BOOL)initials
{
  int pos, x, y, randpl = 0;

  for(x=0; x<9; x++){
    for(y=0; y<9; y++){
      initial[x][y] = NO;
    }
  }

  while(randpl<RAND_DIST){
    int lx = lrand48()%9, ly = lrand48()%9;

    if(initial[lx][ly]==NO){
      initial[lx][ly] = YES;
      randpl++;
    }
  }

  for(x=0; x<9; x++){
    int count = 0;
    for(pos=0; pos<9; pos++){
      if(initial[x][pos]==YES){
	count++;
      }
    }

    if(count>=8){
      return NO;
    }
  }

  for(y=0; y<9; y++){
    int count = 0;
    for(pos=0; pos<9; pos++){
      if(initial[pos][y]==YES){
	count++;
      }
    }

    if(count>=8){
      return NO;
    }
  }
      

  for(x=0; x<3; x++){
    for(y=0; y<3; y++){
      int e, f, count = 0;

      for(e=0; e<3; e++){
	for(f=0; f<3; f++){
	  if(initial[3*x+e][3*y+f]==YES){
	    count++;
	  }
	}
      }

      if(count>=8){
	return NO;
      }
    }
  }

  return YES;
}


- (BOOL)find:(unsigned long)seed
{
  srand48(seed);
 
  if([self initials]==NO){
    return NO;
  }

  success = NO;
  placed = 0;

  int x, y, pos;


  for(x=0; x<9; x++){
    for(y=0; y<9; y++){
      data[x][y].x = x;
      data[x][y].y = y;

      data[x][y].value = -1;
      data[x][y].guess = -1;
      data[x][y].puzzle = -1;

      for(pos=0; pos<9; pos++){
	  data[x][y].nbdigits[pos] = NULL;
      }
    }
  }

  NSAutoreleasePool *pool = [NSAutoreleasePool new];
  NSMutableSet *set = [NSMutableSet setWithCapacity:128];

  NS_DURING

      [self doFind:set];

  NS_HANDLER

      if([[localException name] isEqualToString:EX_COMPLETE]==YES){
	  success = YES;
      }

  NS_ENDHANDLER

      RELEASE(pool);
      
  for(x=0; x<9; x++){
    for(y=0; y<9; y++){
      [self computescore:&(data[x][y])];
    }
  }

  return success;
}

int comparefieldptrs(const void *a, const void *b)
{
  fieldptr 
      ap = *(fieldptr *)a, 
      bp = *(fieldptr *)b;

  if(ap->score < bp->score){
      return +1;
  }
  else if(ap->score > bp->score){
      return -1;
  }
  
  return 0;
}

- doFind:(NSMutableSet *)seen
{
    NSString
      *dataStr = [self stateToString:FIELD_VALUE];
    // *scoreStr = [self stateToString:FIELD_SCORE];

  if(placed==9*9){
    // NSLog(@"--\n%@", dataStr);
    // NSLog(@"--\n%@", scoreStr);
 
      [NSException raise:EX_COMPLETE format:EX_COMPLETE_FMT];
      return self; // not reached
  }

    if([seen containsObject:dataStr]==YES){
	[NSException raise:EX_LOOP format:EX_LOOP_FMT];
	return self; // not reached
    }
    [seen addObject:dataStr];

  int rest = 9*9-placed;

  fieldptr buf[9*9], *bPtr = buf;
  int x, y;

  for(x=0; x<9; x++){
      for(y=0; y<9; y++){
	  fieldptr loc = &(data[x][y]);

	  [self computescore:loc];
          if(data[x][y].value == -1){
              *bPtr++ = loc;
          }
      }
  }
  qsort(buf, rest, sizeof(fieldptr), comparefieldptrs);
  // NSLog(@"--\n%@", dataStr);
  // NSLog(@"--\n%@", scoreStr);

  int score = buf[0]->score, range = 1, mx = 0;
  while(range<rest && buf[range]->score==score){
    range++;
  }

  if(range>1){ // permute
    for(mx=range; mx>1; mx--){
      int ind = lrand48() % mx;

      fieldptr tmp;

      tmp = buf[ind];
      buf[ind] = buf[mx-1];
      buf[mx-1] = tmp;
    }
  }
  
  int pos, d;
  for(pos=0; pos<rest; pos++){
    fieldptr fp = buf[pos];
    int x = fp->x, y = fp->y;

    for(d=0; d<9; d++){
        BOOL perm = YES;
        if(placed<RAND_DIST && initial[x][y]==NO){
	  perm = NO;
	}

	if(perm==YES && fp->nbdigits[d]==NULL){
	  seq[placed].x = x;
	  seq[placed].y = y;

	    placed++;
	    data[x][y].value = d;

	    int nb;
	    for(nb=0; nb<NBCOUNT; nb++){
		int nbx = data[x][y].adj[nb].nx,
		    nby = data[x][y].adj[nb].ny;
		if(data[nbx][nby].nbdigits[d] == NULL){
		    data[nbx][nby].nbdigits[d] = buf;
		}
	    }

            [self doFind:seen];
	
	    for(nb=0; nb<NBCOUNT; nb++){
		int nbx = data[x][y].adj[nb].nx,
		    nby = data[x][y].adj[nb].ny;
		if(data[nbx][nby].nbdigits[d] == buf){
		    data[nbx][nby].nbdigits[d] = NULL;
		}
	    }

	    data[x][y].value = -1;
	    placed--;
	}
    }
  }


  // [seen removeObject:dataStr];
  return self;
}

- (void)drawString:(char *)str atX:(int)x andY:(int)y 
	     color:(NSColor *)col
{

  NSFont *font = [NSFont boldSystemFontOfSize:FONT_SIZE];
  NSDictionary *attrDict =
    [NSDictionary dictionaryWithObjectsAndKeys:
		    font, NSFontAttributeName, 
		  col, NSForegroundColorAttributeName, nil];
  [font set];

  NSString *strObj = [NSString stringWithCString:str];
  NSSize strSize = [strObj sizeWithAttributes:attrDict];	    

  NSPoint loc =
    NSMakePoint(x*FIELD_DIM + (FIELD_DIM-strSize.width)/2,
		(8-y)*FIELD_DIM + (FIELD_DIM-strSize.height)/2);
  [strObj drawAtPoint:loc withAttributes:attrDict];
}

- (void)drawRect:(NSRect)rect
{
    [[NSColor whiteColor] set];
    NSRectFill([self bounds]);

    // NSRect frect = [font boundingRectForFont];

    int x, y, pos;

    PSsetlinewidth(2);
    PSsetrgbcolor(0, 0, 0);

    for(pos=0; pos<=9; pos++){
	if(!(pos % 9)){
	    PSsetlinewidth(12);
	}
	else if(!(pos % 3)){
	    PSsetlinewidth(6);
	}
	else{
	    PSsetlinewidth(3);
	}

	PSmoveto(0, pos*FIELD_DIM);
	PSlineto(9*FIELD_DIM, pos*FIELD_DIM);
	PSstroke();

	PSmoveto(pos*FIELD_DIM, 0);
	PSlineto(pos*FIELD_DIM, 9*FIELD_DIM);
	PSstroke();
    }

    for(x=0; x<9; x++){
        for(y=0; y<9; y++){
	    NSColor *col;
	    char str[2] = { '1', 0 };

	    if(data[x][y].puzzle != -1){
		col = [NSColor blackColor];
		str[0] += data[x][y].puzzle;
	    }
	    else if(data[x][y].guess != -1){
		col = [NSColor blueColor];
		str[0] += data[x][y].guess;
	    }
	    else{
		col = [NSColor blackColor];
		str[0] = '.';
	    }

	    [self drawString:str atX:x andY:y color:col];
	}
    }
}

- (int)setClues:(int)count
{
    int prev = clues;

    int x, y;
    for(x=0; x<9; x++){
	for(y=0; y<9; y++){
	  data[x][y].puzzle = -1;
	  data[x][y].guess = -1;
	}
    }

    int s;
    // for(s=9*9-count; s<9*9; s++){
    for(s=0; s<count; s++){
      int x = seq[s].x, y = seq[s].y;
      data[x][y].puzzle = data[x][y].value;
    }

    clues = count;
    return prev;
}

- (int)clues
{
    return clues;
}

- reset
{
  int x, y;

  for(x=0; x<9; x++){
    for(y=0; y<9; y++){
      data[x][y].guess = -1;
    }
  }

  [self setNeedsDisplay:YES];
  [self display];

  [[self window] flushWindow];

  return self;
}  


#define TICK 0.75
#define POLL 0.20

- (void)mouseDown:(NSEvent *)theEvent
{
  NSPoint startp = [theEvent locationInWindow];
  startp = [self convertPoint:startp fromView: nil];
    
  NSWindow *win = [self window];
  NSEvent *curEvent = theEvent;

  unsigned int cmask = NSLeftMouseUpMask | NSLeftMouseDraggedMask;

  startp.x/=(float)FIELD_DIM; startp.y/=(float)FIELD_DIM; 
  int x = (int)startp.x, y = 8-(int)startp.y;

  if(data[x][y].puzzle != -1){
    NSBeep();
    return;
  }

  int guess = data[x][y].guess;

  NSRect box = 
    NSMakeRect(x*FIELD_DIM, (8-y)*FIELD_DIM, 
	       FIELD_DIM,  FIELD_DIM), cachebox = box;
  cachebox.origin.x -= 4;
  cachebox.origin.y -= 4;
  cachebox.size.width += 8;
  cachebox.size.height += 8;
  
  cachebox = [self convertRect:cachebox toView:nil];

  BOOL first = YES;
  
  NSDate *tick = [NSDate dateWithTimeIntervalSinceNow:TICK];

  NSBezierPath 
    *bpfilled = [NSBezierPath bezierPathWithRect:box],
    *bpstroked = [NSBezierPath bezierPathWithRect:box];

  [bpstroked setLineWidth:6.0];

  guess = 
      (guess==-1 ? 0 : (guess==8 ? -1 : guess+1));

  do {
    if(first==NO){
      [win restoreCachedImage];
    }
    first = NO;       

    [win cacheImageInRect:cachebox];

    [self lockFocus];

    [[NSColor greenColor] set];
    [bpfilled fill];

    [[NSColor blueColor] set];
    [bpstroked stroke];

    NSDate *now = [NSDate date];
    if([now laterDate:tick]==now){
      guess = 
	(guess==-1 ? 0 : (guess==8 ? -1 : guess+1));
      tick = [NSDate dateWithTimeIntervalSinceNow:TICK];
    }

    char str[2] = { 
      (guess==-1 ? '.' : '1' + guess), 
      0 };
    [self drawString:str atX:x andY:y 
     color:[NSColor blackColor]];

    [self unlockFocus];

    [win flushWindow];

    curEvent = 
      [[self window]
	nextEventMatchingMask:cmask
	untilDate:[NSDate dateWithTimeIntervalSinceNow:POLL]
	inMode:NSEventTrackingRunLoopMode
	dequeue:YES];
  } while(curEvent==nil || [curEvent type] != NSLeftMouseUp);

  [win restoreCachedImage];
  [win flushWindow];
        
  if(guess == data[x][y].guess){
    return;
  }
  
  NSDocumentController *dc =
      [NSDocumentController sharedDocumentController];

  if(guess == -1){
    data[x][y].guess = -1;

    [self setNeedsDisplay:YES];

    [self display];
    [win flushWindow];

    [[dc currentDocument] updateChangeCount:NSChangeDone];
    return;
  }

  char str[2] = { '1' + guess, 0 };

  int nb;
  for(nb=0; nb<NBCOUNT; nb++){
    int 
      nx = data[x][y].adj[nb].nx, 
      ny = data[x][y].adj[nb].ny;

    if(RETR(nx, ny)==guess){
      break;
    }
  }

  if(nb<NBCOUNT){
    NSBeep();
    
    [win cacheImageInRect:cachebox];

    [self lockFocus];

    [[NSColor redColor] set];
    [bpfilled fill];

    [[NSColor blueColor] set];
    [bpstroked stroke];

    [self drawString:str atX:x andY:y
     color:[NSColor blackColor]];

    [self unlockFocus];

    [win flushWindow];

    NSDate *errpause =
      [NSDate dateWithTimeIntervalSinceNow:TICK*5];
    [NSThread sleepUntilDate:errpause];

    [win restoreCachedImage];
    [win flushWindow];

    return;
  }

  [[dc currentDocument] updateChangeCount:NSChangeDone];

  data[x][y].guess = guess;
  [self setNeedsDisplay:YES];

  [self display];
  [win flushWindow];

  if([self completed]==YES){
    NSRunAlertPanel(@"Congratulations!", @"Sudoku solved.",
		    @"Ok", nil, nil);
  }
    
  return;
}

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
  NSPasteboard *pb = [sender draggingPasteboard];
  NSDragOperation
    sourceDragMask = [sender draggingSourceOperationMask];

  if([[pb types] indexOfObject:DIGIT_TYPE]!=NSNotFound){
    if(sourceDragMask & NSDragOperationCopy){
      NSString *dstr = [pb stringForType:DIGIT_TYPE];

      int newDigit;
      [[NSScanner scannerWithString:dstr] scanInt:&newDigit];

      NSPoint startp = [sender draggingLocation];

      startp.x/=(float)FIELD_DIM; startp.y/=(float)FIELD_DIM; 
      int x = (int)startp.x, y = 8-(int)startp.y;

      if(data[x][y].puzzle != -1){
	return NSDragOperationNone;
      }

      if(newDigit==10){
	return NSDragOperationCopy;
      }

      newDigit--;

      int nb;
      for(nb=0; nb<NBCOUNT; nb++){
	int 
	  nx = data[x][y].adj[nb].nx, 
	  ny = data[x][y].adj[nb].ny;

	if(RETR(nx, ny)==newDigit){
	  break;
	}
      }

      if(nb<NBCOUNT){
	return NSDragOperationNone;
      }
    }
  }
 
  return NSDragOperationCopy;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
  return [self draggingEntered:sender];
}


- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
  NSPasteboard *pb = [sender draggingPasteboard];
  NSString *dstr = [pb stringForType:DIGIT_TYPE];
  
  int newDigit;
  [[NSScanner scannerWithString:dstr] scanInt:&newDigit];

  NSPoint startp = [sender draggingLocation];

  startp.x/=(float)FIELD_DIM; startp.y/=(float)FIELD_DIM; 
  int x = (int)startp.x, y = 8-(int)startp.y;

  data[x][y].guess = 
    (newDigit==10 ? -1 : newDigit-1);

  [self setNeedsDisplay:YES];
  [self display];
  [[self window] flushWindow];

  NSDocumentController *dc =
      [NSDocumentController sharedDocumentController];
  [[dc currentDocument] updateChangeCount:NSChangeDone];

  if([self completed]==YES){
    NSRunAlertPanel(@"Congratulations!", @"Sudoku solved.",
		    @"Ok", nil, nil);
  }

  return YES;
}



- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
  return YES;
}


@end
